/*
 * myi2c.c
 *
 *  Created on: 08.03.2018
 *      Author: Procesorowiec
 */


#include "system8xx.h"
#include "myi2c.h"



#define I2C_PORT			0
#define I2C_SCL_PIN			10
#define I2C_SDA_PIN			11

#define MYI2C				LPC_I2C0
#define MYI2C_CLOCK			MAIN_CLOCK
#define MYI2C_CLK_DIV       40
#define MYI2C_BITRATE       100000


typedef struct {
	uint8_t devAddr;
	uint8_t *sendSrc;
	uint16_t sendLen;
	uint8_t *recDst;
	uint16_t recLen;
	volatile uint8_t status;
} MYI2C_TRANSFER_T;


MYI2C_TRANSFER_T myI2cTransferReg;


static void MyI2cTransferStart(MYI2C_TRANSFER_T *transferReg);
static void MyI2cSetBusSpeed(uint32_t busSpeed);
static void MyI2cHandler(MYI2C_TRANSFER_T *transferReg);


//------------------------------------------------------
void MyI2cInit(void){

	Chip_IOCON_PinSetI2CMode(LPC_IOCON, IOCON_PIO10, PIN_I2CMODE_STDFAST);
	Chip_IOCON_PinSetI2CMode(LPC_IOCON, IOCON_PIO11, PIN_I2CMODE_STDFAST);

	Chip_GPIO_SetPortDIROutput(LPC_GPIO_PORT, I2C_PORT, (1<<I2C_SCL_PIN | 1<<I2C_SDA_PIN));
	Chip_GPIO_SetPortOutHigh(LPC_GPIO_PORT, I2C_PORT, (1<<I2C_SCL_PIN | 1<<I2C_SDA_PIN));

	Chip_SWM_EnableFixedPin(SWM_FIXED_I2C0_SDA);
	Chip_SWM_EnableFixedPin(SWM_FIXED_I2C0_SCL);

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_I2C0);
	Chip_SYSCTL_PeriphReset(RESET_I2C0);

	Chip_I2C_SetClockDiv(MYI2C, MYI2C_CLK_DIV);
	MyI2cSetBusSpeed(MYI2C_BITRATE);

	/* Enable Master Mode */
	Chip_I2CM_Enable(MYI2C);
	NVIC_EnableIRQ(I2C0_IRQn);

}

//------------------------------------------------------
static void MyI2cSetBusSpeed(uint32_t busSpeed){
	uint32_t scl = (MYI2C_CLOCK	 / (MYI2C_CLK_DIV * busSpeed));
	uint16_t sclH = (scl >> 1);
	uint16_t sclL = (scl - (scl >> 1));

	if (sclH < 2) {
		sclH = 2;
	}
	else if (sclH > 9) {
		sclH = 9;
	}
	if (sclL < 2) {
		sclL = 2;
	}
	else if (sclL > 9) {
		sclL = 9;
	}

	MYI2C->MSTTIME = (((sclH - 2) & 0x07) << 4) | ((sclL - 2) & 0x07);
}

//------------------------------------------------------
void I2C0_IRQHandler(void){

	MyI2cHandler(&myI2cTransferReg);
	return;
}

//------------------------------------------------------
uint8_t MyI2cTransfer(uint8_t devAddr,
		   	   	   uint8_t *sendSrc,
				   uint16_t sendLen,
				   uint8_t *recDst,
				   uint16_t recLen){

	myI2cTransferReg.devAddr = devAddr;
	myI2cTransferReg.sendSrc = sendSrc;
	myI2cTransferReg.sendLen = sendLen;
	myI2cTransferReg.recDst = recDst;
	myI2cTransferReg.recLen = recLen;

	uint32_t i = 0;
	MyI2cTransferStart(&myI2cTransferReg);

	while (myI2cTransferReg.status != I2CM_STATUS_OK){
		i++;
		if (i > 1000000){
			//SerialDebugNewReport();
			SerialDebugAddParam("i2c err", myI2cTransferReg.status);
			//SerialDebugSendReport();
			myI2cTransferReg.status = I2CM_STATUS_ERROR;
			break;
		}
	}

	Chip_I2C_ClearInt(MYI2C, I2C_INTENSET_MSTPENDING | I2C_INTENSET_MSTRARBLOSS | I2C_INTENSET_MSTSTSTPERR);

	return myI2cTransferReg.status;
	//return I2CM_STATUS_OK;
}

//------------------------------------------------------
static void MyI2cTransferStart(MYI2C_TRANSFER_T *transferReg){
	transferReg->status = I2CM_STATUS_BUSY;

	Chip_I2CM_ClearStatus(MYI2C, I2C_STAT_MSTRARBLOSS | I2C_STAT_MSTSTSTPERR);

	if (transferReg->sendLen == 0){
		Chip_I2CM_WriteByte(MYI2C, (transferReg->devAddr | 0x01));
	} else {
		Chip_I2CM_WriteByte(MYI2C, (transferReg->devAddr & 0xFE));
	}
	Chip_I2CM_SendStart(MYI2C);
	Chip_I2C_EnableInt(MYI2C, I2C_INTENSET_MSTPENDING | I2C_INTENSET_MSTRARBLOSS | I2C_INTENSET_MSTSTSTPERR);
}

//------------------------------------------------------
static void MyI2cHandler(MYI2C_TRANSFER_T *transferReg){
	uint32_t status = Chip_I2CM_GetStatus(MYI2C);

	/* Master Lost Arbitration */
	if (status & I2C_STAT_MSTRARBLOSS) {
		transferReg->status = I2CM_STATUS_ARBLOST;
		Chip_I2CM_ClearStatus(MYI2C, I2C_STAT_MSTRARBLOSS);
	}

	/* Master Start Stop Error */
	else if (status & I2C_STAT_MSTSTSTPERR) {
		transferReg->status = I2CM_STATUS_BUS_ERROR;
		Chip_I2CM_ClearStatus(MYI2C, I2C_STAT_MSTSTSTPERR);
	}

	/* Master is Pending */
	else if (status & I2C_STAT_MSTPENDING) {

		/* Branch based on Master State Code */
		switch (Chip_I2CM_GetMasterState(MYI2C)) {

			/* Master idle */
			case I2C_STAT_MSTCODE_IDLE:
				/* Master job is done, disable this IRQ */
				Chip_I2C_ClearInt(MYI2C, I2C_INTENSET_MSTPENDING);
				transferReg->status = I2CM_STATUS_OK;
				break;

			/* Receive data is available */
			case I2C_STAT_MSTCODE_RXREADY:
				/* Read Data */
				*transferReg->recDst++ = MYI2C->MSTDAT;
				//transferReg->recDst++;
				transferReg->recLen--;
				if (transferReg->recLen > 0) {
					Chip_I2CM_MasterContinue(MYI2C);
				} else {
					//transferReg->status = I2CM_STATUS_OK;
					Chip_I2CM_SendStop(MYI2C);
				}
				break;

			/* Master Transmit available */
			case I2C_STAT_MSTCODE_TXREADY:
				if (transferReg->sendLen > 0) {
					/* If Tx data available transmit data and continue */
					MYI2C->MSTDAT = *transferReg->sendSrc++;
					//transferReg->sendSrc++;
					transferReg->sendLen--;
					Chip_I2CM_MasterContinue(MYI2C);
				} else {
					/* If receive queued after transmit then initiate master receive transfer*/
					if (transferReg->recLen > 0) {
						Chip_I2CM_WriteByte(MYI2C, (transferReg->devAddr | 0x01));
						Chip_I2CM_SendStart(MYI2C);
					} else {
						/* If no receive queued then set transfer status as OK */
						//transferReg->status = I2CM_STATUS_OK;
						Chip_I2CM_SendStop(MYI2C);
					}
				}
				break;

			case I2C_STAT_MSTCODE_NACKADR:
				/* Set transfer status as NACK on address */
				transferReg->status = I2CM_STATUS_NAK_ADR;
				Chip_I2CM_SendStop(MYI2C);
				break;

			case I2C_STAT_MSTCODE_NACKDAT:
				/* Set transfer status as NACK on data */
				transferReg->status = I2CM_STATUS_NAK_DAT;
				Chip_I2CM_SendStop(MYI2C);
				break;

			default:
				/* Default case should not occur*/
				transferReg->status = I2CM_STATUS_ERROR;
				break;
		}
	}
	else {
		/* Default case should not occur */
		transferReg->status = I2CM_STATUS_ERROR;
	}
	//return transferReg->status != I2CM_STATUS_BUSY;
}

